// Driver for SAA1064 i2c to non-multiplexed two-digit, seven-segment led display
// Modified for kello v3 by JKA.
#define byte int8 // Listty ett kntyy ja toimii

// SAA1064 write addresses set by connecting pin1 (ADR) to different voltages
#define LED_ADDR0       0x70  // ADR to Vee
#define LED_ADDR1       0x72  // ADR to 3/8 Vcc
#define LED_ADDR2       0x74  // ADR to 5/8 Vcc
#define LED_ADDR3       0x76  // ADR to Vcc

// Control bits
// C0 = 0 = static mode  C0 = 1 dynamic mode
// C1 = 0/1 digits 1 and 3 are blanked/not blanked
// C2 = 0/1 digits 2 and 4 are blanked/not blanked
// C3 = 1 all segments outputs turned on for segment test
// C4 = 1 adds 3ma to segment output
// C5 = 1 adds 6ma to segment output
// C6 = 1 adds 12ma to segment output
// C7 n/a

// #define LED_CONTROL 0b00100110  // Static mode 6mA
// #define LED_CONTROL 0b00010110  // Static mode 3mA
byte LED_CONTROL = 0b00010110;

// Segment table (In hex: d.p.=80,g=40,f=20,e=10,d=08,c=04,b=02,a=01)
byte const seg_table[10]= {0b11100111,0b10000001,0b01101011,0b11001011,0b10001101,
 0b11001110,0b11101110,0b10000011,0b11101111,0b11001111}; // 0,1,2,3,4,5,6,7,8,9

byte const seg_table_right[10]= {0b11111010,0b00110000,0b11011001,0b01111001,0b00110011,
 0b01101011,0b11101011,0b00111000,0b11111011,0b01111011}; // 0,1,2,3,4,5,6,7,8,9

//--------------------------------------------------------------------------
// Sends number to two-digit led display via SAA1064  (0 to 99)
void send_saa1064_num(int8 addr,int8 control,int8 num, int8 dps)
{
   int8 j;                       // Digit index
   int8 led_data[2];     // Ram array for segment data
   int8 temp8;
   
   // Get led segments for all two digits. Don't need to divide to get units.
   temp8 = num; 

   led_data[0] = seg_table_right[temp8 %10];
   temp8 /= 10;
   led_data[1] = seg_table[temp8 %10];


   // Do leading zero blanking
   // if (num<1000) led_data[3] = 0x00;
   // if (num<100)  led_data[2] = 0x00;
   // if (num<10)   led_data[1] = 0x00;

   // Add decimal point if required
   // if (num>99)led_data[1] |= 0x10;
   if (dps==2)
      led_data[1] |= 0x10;
   if (dps==1)
      led_data[0] |= 0x04;

   // Send led segments
   i2c_start();
   i2c_write(addr);        // Send SAA1064 address
   i2c_write(0x00);        // Send instruction byte. Zero is control reg.
   i2c_write(control);     // Send control byte

   // Auto increment applies for digit data. Have to send out digits in reverse
   // because most significant digit is 1st digit.
   for (j=2;j>=1;j--)           
      i2c_write(led_data[j-1] & 0xff);   
    
   i2c_stop();
}
//---------------------------------------------------------------------------
//Sends raw segment data to led display via SAA1064

void send_saa1064_raw(int8 addr,int8 digit,int8 value)
{
   // Send led segment
   i2c_start();
   i2c_write(addr);      // Send  SAA1064 address
   i2c_write(digit+1);   // Send instruction byte. Zero is control reg, so digit is 1+control
   i2c_write(value);     // Send segment data
   i2c_stop();
}

void set_saa1064_current(int8 current)
{ 
	switch(current)
	{
		case 3:
			LED_CONTROL = 0b00010110;
			break;
		case 6:
			LED_CONTROL = 0b00100110;
			break;
		case 9:
			LED_CONTROL = 0b00110110;
			break;
		case 12:
			LED_CONTROL = 0b01000110;
			break;
		case 15:
			LED_CONTROL = 0b01010110;
			break;
	}	
	
	i2c_start();
	i2c_write(LED_ADDR0);      // Send  SAA1064 address
	i2c_write(0x00);     // Send instruction byte. Zero is control reg.
	i2c_write(LED_CONTROL);  // Send control byte
	i2c_stop();

	i2c_start();
	i2c_write(LED_ADDR3);      // Send  SAA1064 address
	i2c_write(0x00);     // Send instruction byte. Zero is control reg.
	i2c_write(LED_CONTROL);  // Send control byte
	i2c_stop();

	i2c_start();
	i2c_write(LED_ADDR1);      // Send  SAA1064 address
	i2c_write(0x00);     // Send instruction byte. Zero is control reg.
	i2c_write(LED_CONTROL);  // Send control byte
	i2c_stop();
}			